@@ -1,12 +1,3 @@
-#
-# This file is part of CPAN-Reporter
-#
-# This software is Copyright (c) 2006 by David Golden.
-#
-# This is free software, licensed under:
-#
-#   The Apache License, Version 2.0, January 2004
-#
 package t::Helper;
 use strict;
 BEGIN{ if (not $] < 5.006) { require warnings; warnings->import } }
@@ -21,22 +12,18 @@ use vars qw/@EXPORT/;
     test_dispatch test_dispatch_plan
 /;
 
-use Exporter ();
-our @ISA = 'Exporter';
+use base 'Exporter';
 
 use Config;
-use Archive::Tar 1.54 ();
 use File::Basename qw/basename/;
-use File::Copy::Recursive 0.35 qw/dircopy/;
+use File::Copy::Recursive qw/dircopy/;
 use File::Path qw/mkpath/;
-use File::pushd 0.32 qw/pushd tempd/;
-use File::Spec 3.19 ();
-use File::Temp 0.16 qw/tempdir/;
-use IO::CaptureOutput 1.03 qw/capture/;
+use File::pushd qw/tempd/;
+use File::Spec ();
+use File::Temp qw/tempdir/;
+use IO::CaptureOutput qw/capture/;
 use Probe::Perl ();
-use Test::More 0.62;
-
-use t::MockHomeDir;
+use Test::More;
 
 #--------------------------------------------------------------------------#
 # Fixtures
@@ -45,38 +32,31 @@ use t::MockHomeDir;
 my $perl = Probe::Perl->find_perl_interpreter();
 my $make = $Config{make};
 
-my $temp_stdout = File::Temp->new()
+my $temp_stdout = File::Temp->new() 
     or die "Couldn't make temporary file:$!\nIs your temp drive full?";
-
-my $temp_dist_dir = tempdir( 'CR-t-dist-XXXXXX', CLEANUP => 1, TMPDIR => 1);
-my $dist_archive = File::Spec->rel2abs("t/dist.tgz");
-{
-  my $wd = pushd($temp_dist_dir);
-  Archive::Tar->extract_archive($dist_archive, 1)
-    or die "Could not extract test distributions: " . Archive::Tar->error;
-}
-
-my $home_dir = t::MockHomeDir::home_dir();
+my $temp_home = tempdir(
+        "CPAN-Reporter-testhome-XXXXXXXX", TMPDIR => 1, CLEANUP => 1
+) or die "Couldn't create temporary config dir: $!\nIs your temp drive full?";
+my $home_dir = File::Spec->rel2abs( $temp_home );
 my $config_dir = File::Spec->catdir( $home_dir, ".cpanreporter" );
 my $config_file = File::Spec->catfile( $config_dir, "config.ini" );
 
 my $bogus_email_from = 'johndoe@example.com';
 my $bogus_email_to = 'no_one@example.com';
+my $bogus_smtp = 'mail.mail.com';
 
 my %tool_constants = (
     'eumm'  => {
         module  => 'ExtUtils::MakeMaker',
-        have    => eval "require ExtUtils::MakeMaker" || 0,
         PL      => 'Makefile.PL',
     },
     'mb'    => {
         module  => 'Module::Build',
-        have    => eval "require Module::Build" || 0,
         PL      => 'Build.PL'
     },
 );
 
-my $max_report_length = 1_000_000; # 1000K
+my $max_report_length = 50_000; # 50K
 
 # used to capture from fixtures
 use vars qw/$sent_report @cc_list/;
@@ -85,35 +65,31 @@ use vars qw/$sent_report @cc_list/;
 # test config file prep
 #--------------------------------------------------------------------------#
 
-sub test_fake_config_plan() { 4 }
+sub test_fake_config_plan() { 3 }
 sub test_fake_config {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my %overrides = @_;
 
-    is( File::HomeDir::my_documents(), t::MockHomeDir::home_dir(),
+    is( File::HomeDir::my_documents(), $home_dir,
         "home directory mocked"
-    );
+    ); 
     mkpath $config_dir;
     ok( -d $config_dir,
         "config directory created"
     );
-    my $metabase_file = File::Spec->catfile( $config_dir, 'metabase_id.json' );
-    # 2-args open with bare descriptor to work in older perls
-    open METABASE, ">$metabase_file";
-    close METABASE;
-    ok -r $metabase_file, 'created mock metabase file for testing';
 
     my $tiny = Config::Tiny->new();
     $tiny->{_}{email_from} = $bogus_email_from;
     $tiny->{_}{email_to} = $bogus_email_to; # failsafe
+    $tiny->{_}{smtp_server} = $bogus_smtp;
+    $tiny->{_}{cc_author} = "yes";
     $tiny->{_}{send_report} = "yes";
     $tiny->{_}{send_duplicates} = "yes"; # tests often repeat same stuff
-    $tiny->{_}{transport} = "Metabase uri https://metabase.cpantesters.org/api/v1/ id_file metabase_id.json";
     for my $key ( keys %overrides ) {
         $tiny->{_}{$key} = $overrides{$key};
     }
     ok( $tiny->write( $config_file ),
-        "created temp config file"
+        "created temp config file with a new email address and smtp server"
     );
 }
 
@@ -122,7 +98,7 @@ sub test_fake_config {
 #--------------------------------------------------------------------------#
 
 sub test_grade_PL_iter_plan() { 5 }
-sub test_grade_PL_plan() { test_grade_PL_iter_plan() * 2 }
+sub test_grade_PL_plan() { test_grade_PL_iter_plan() * 2 } 
 sub test_grade_PL {
     my ($case, $dist) = @_;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -131,27 +107,27 @@ sub test_grade_PL {
 
     for my $tool ( qw/eumm mb/ ) {
         SKIP: {
-            my ($have_tool,$tool_mod,$tool_PL) =
-                @{$tool_constants{$tool}}{qw/have module PL/};
-            my $tool_label = $tool eq 'eumm'  ? "Makefile.PL"
+            my ($tool_mod,$tool_PL) = @{$tool_constants{$tool}}{qw/module PL/};
+            my $tool_label = $tool eq 'eumm'  ? "Makefile.PL" 
                                             : "Build.PL";
-            my $tool_cmd = "$perl $tool_label";
-
-            skip "$tool_mod not installed or working", test_grade_PL_iter_plan()
-                if ! $have_tool;
+            my $tool_cmd = "$perl $tool_label"; 
 
+            eval "require $tool_mod";
+            skip "$tool_mod not installed", test_grade_PL_iter_plan()
+                if $@;
+            
             my $tempd = _ok_clone_dist_dir( $case->{name} );
-            local $dist->{build_dir} = "$tempd";
-
+            
             $t::Helper::sent_report = undef;
             $t::Helper::comments = undef;
+            @t::Helper::cc_list = ();
 
-            my ($stdout, $stderr, $build_rc, $test_build_rc,
+            my ($stdout, $stderr, $build_rc, $test_build_rc, 
                 $output, $exit_value, $rc);
 
             eval {
                 capture sub {
-                    ($output, $exit_value) =
+                    ($output, $exit_value) = 
                         CPAN::Reporter::record_command($tool_cmd);
                     $rc = CPAN::Reporter::grade_PL(
                         $dist, $tool_cmd, $output, $exit_value
@@ -163,28 +139,28 @@ sub test_grade_PL {
                 _diag_output( $stdout, $stderr );
                 skip "died grading PL", test_grade_PL_iter_plan() - 1;
             }
-
-            my $is_rc_correct = $case->{"$tool\_success"}
+            
+            my $is_rc_correct = $case->{"$tool\_success"} 
                               ? $rc : ! $rc;
 
-            ok( $is_rc_correct,
-                "$case->{name}: grade_PL() for $tool_PL returned " .
+            ok( $is_rc_correct, 
+                "$case->{name}: grade_PL() for $tool_PL returned " . 
                 $case->{"$tool\_success"}
             );
-
+            
             my $case_grade = $case->{"$tool\_grade"};
             my $is_grade_correct;
 
             # Grade evaluation with special case if discarding
-            my ($found_grade_result, $found_msg) =
+            my ($found_grade_result, $found_msg) = 
                 ( $stdout =~ /^CPAN::Reporter: ([^,]+), ([^\n]+)/ms );
             if ( $case_grade eq 'discard' ) {
-                is ($found_grade_result, "test results were not valid",
-                    "$case->{name}: '$tool_label' saw test results not valid message"
+                is ($found_grade_result, "Test results were not valid",
+                    "$case->{name}: '$tool_label' prerequisites not satisifed"
                 );
-
-                like( $stdout,
-                    "/Test report will not be sent/",
+                    
+                like( $stdout, 
+                    "/Test results for \Q$short_name\E will be discarded/",
                     "$case->{name}: discard message correct"
                 ) and $is_grade_correct++;
 
@@ -194,12 +170,12 @@ sub test_grade_PL {
             }
             else {
                 my ($found_grade) = ( $found_grade_result =~ /$tool_label result is '([^']+)'/ );
-                is( $found_grade, $case_grade,
+                is( $found_grade, $case_grade, 
                     "$case->{name}: '$tool_label' grade reported as '$case_grade'"
                 ) or _diag_output( $stdout, $stderr );
-
-                my $ctr_regex = "/preparing a CPAN Testers report for \Q$short_name\E/";
-
+                
+                my $ctr_regex = "/Preparing a CPAN Testers report for \Q$short_name\E/";
+                
                 if ( $case_grade eq 'pass' ) {
                     unlike( $stdout, $ctr_regex ,
                         "$case->{name}: report notification correct"
@@ -225,7 +201,7 @@ sub test_grade_PL {
                 }
             }
 
-            _diag_output( $stdout, $stderr )
+            _diag_output( $stdout, $stderr ) 
                 unless ( $is_rc_correct && $is_grade_correct );
         } # SKIP
     } # for
@@ -235,8 +211,8 @@ sub test_grade_PL {
 # Test grade_make
 #--------------------------------------------------------------------------#
 
-sub test_grade_make_iter_plan() { 6 }
-sub test_grade_make_plan() { test_grade_make_iter_plan() * 2 }
+sub test_grade_make_iter_plan() { 7 }
+sub test_grade_make_plan() { test_grade_make_iter_plan() * 2 } 
 sub test_grade_make {
     my ($case, $dist) = @_;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -245,21 +221,22 @@ sub test_grade_make {
 
     for my $tool ( qw/eumm mb/ ) {
         SKIP: {
-            my ($have_tool,$tool_mod,$tool_PL) =
-                @{$tool_constants{$tool}}{qw/have module PL/};
+            my ($tool_mod,$tool_PL) = @{$tool_constants{$tool}}{qw/module PL/};
             my $tool_cmd = $tool eq 'eumm' ? $Config{make} : "$perl Build";
             my $tool_label = $tool eq 'eumm' ? $Config{make} : "Build";
 
-            skip "$tool_mod not installed or working", test_grade_make_iter_plan()
-                if ! $have_tool;
-
+            eval "require $tool_mod";
+            skip "$tool_mod not installed", test_grade_make_iter_plan()
+                if $@;
+            
             # Set up temporary directory for the case
             my $tempd = _ok_clone_dist_dir( $case->{name} );
 
             $t::Helper::sent_report = undef;
             $t::Helper::comments = undef;
+            @t::Helper::cc_list = ();
 
-            my ($stdout, $stderr, $build_err, $test_build_rc,
+            my ($stdout, $stderr, $build_err, $test_build_rc, 
                 $output, $exit_value, $rc);
 
             capture sub {
@@ -273,7 +250,7 @@ sub test_grade_make {
                 };
             eval {
                 capture sub {
-                    ($output, $exit_value) =
+                    ($output, $exit_value) = 
                     CPAN::Reporter::record_command($tool_cmd);
                     $rc = CPAN::Reporter::grade_make(
                         $dist, $tool_cmd, $output, $exit_value
@@ -286,72 +263,49 @@ sub test_grade_make {
                 skip "died grading make", test_grade_make_iter_plan() - 1;
             }
 
-            my $is_rc_correct = $case->{"$tool\_success"}
+            my $is_rc_correct = $case->{"$tool\_success"} 
             ? $rc : ! $rc;
 
-            ok( $is_rc_correct,
-                "$case->{name}: grade_make() for $tool_label returned " .
+            ok( $is_rc_correct, 
+                "$case->{name}: grade_make() for $tool_label returned " . 
                 $case->{"$tool\_success"}
             );
 
             my $case_grade = $case->{"$tool\_grade"};
+
+            # correct grade identified?
             my $is_grade_correct;
+            like( $stdout, "/^CPAN::Reporter: \Q$tool_label\E result is '$case_grade'/ms",
+                "$case->{name}: $tool_label grade identified as '$case_grade'"
+            ) and $is_grade_correct++;
+            my $case_msg = $case->{"$tool\_msg"};
+            like( $stdout, "/\Q$case_msg\E/",
+                "$case->{name}: $tool_label grade explanation correct"
+            );
 
-            # Grade evaluation with special case if discarding
-            my ($found_grade_result, $found_msg) =
-                ( $stdout =~ /^CPAN::Reporter: ([^,]+), ([^\n]+)/ms );
-            if ( $case_grade eq 'discard' ) {
-                is ($found_grade_result, "test results were not valid",
-                    "$case->{name}: '$tool_label' prerequisites not satisifed"
+            # expectations are different by grade
+            if ( $case_grade =~ m{fail|unknown|na} ) {
+                # report should have been sent
+                like( $stdout, "/Preparing a CPAN Testers report for \Q$short_name\E/",
+                    "$case->{name}: report notification correct"
                 );
-
-                like( $stdout,
-                    "/Test report will not be sent/",
-                    "$case->{name}: discard message correct"
-                ) and $is_grade_correct++;
-
-                ok( ! defined $t::Helper::sent_report,
-                    "$case->{name}: test results discarded"
+                ok( defined $t::Helper::sent_report && length $t::Helper::sent_report,
+                    "$case->{name}: report was mock sent"
                 );
             }
-            else {
-                my ($found_grade) = ( $found_grade_result =~ /\Q$tool_label\E result is '([^']+)'/ );
-                is( $found_grade, $case_grade,
-                    "$case->{name}: '$tool_label' grade reported as '$case_grade'"
-                ) or _diag_output( $stdout, $stderr );
-
-                my $ctr_regex = "/preparing a CPAN Testers report for \Q$short_name\E/";
-
-                if ( $case_grade eq 'pass' ) {
-                    unlike( $stdout, $ctr_regex ,
-                        "$case->{name}: report notification correct"
-                    ) and $is_grade_correct++;
-                    ok( ! defined $t::Helper::sent_report,
-                        "$case->{name}: results not sent"
-                    );
-                }
-                else {
-                    like( $stdout, $ctr_regex ,
-                        "$case->{name}: report notification correct"
-                    ) and $is_grade_correct++;
-                    if ( -r $config_file ) {
-                        ok( defined $t::Helper::sent_report && length $t::Helper::sent_report,
-                            "$case->{name}: report was mock sent"
-                        );
-                    }
-                    else {
-                        ok( ! defined $t::Helper::sent_report,
-                            "$case->{name}: results not sent"
-                        );
-                    }
-                }
+            else { # pass
+                # report shouldn't have been sent
+                ok( ! defined $t::Helper::sent_report ,
+                    "$case->{name}: no $tool_label report was sent"
+                );
+                pass("$case->{name}: (advancing test count)");
             }
 
             _diag_output( $stdout, $stderr )
                 unless ( $is_rc_correct && $is_grade_correct );
 
         } #SKIP
-    } #for
+    } #for   
 }
 
 #--------------------------------------------------------------------------#
@@ -368,20 +322,21 @@ sub test_grade_test {
 
     for my $tool ( qw/eumm mb/ ) {
         SKIP: {
-            my ($have_tool,$tool_mod,$tool_PL) =
-                @{$tool_constants{$tool}}{qw/have module PL/};
+            my ($tool_mod,$tool_PL) = @{$tool_constants{$tool}}{qw/module PL/};
             my $tool_cmd = $tool eq 'eumm'  ?  "$make test"
                                             :  "$perl Build test" ;
             my $tool_label = $tool eq 'eumm'?  "make test"
                                             :  "perl Build test" ;
 
-            skip "$tool_mod not installed or working", test_grade_test_iter_plan()
-                if ! $have_tool;
+            eval "require $tool_mod";
+            skip "$tool_mod not installed", test_grade_test_iter_plan()
+                if $@;
 
             my $tempd = _ok_clone_dist_dir( $case->{name} );
-
+                
             $t::Helper::sent_report = undef;
             $t::Helper::comments = undef;
+            @t::Helper::cc_list = ();
 
             my ($stdout, $stderr, $build_err, $test_build_rc);
 
@@ -406,23 +361,23 @@ sub test_grade_test {
                 skip "test() died", test_grade_test_iter_plan() - 1;
             }
 
-            my $is_rc_correct = $case->{"$tool\_success"}
+            my $is_rc_correct = $case->{"$tool\_success"} 
                               ? $test_build_rc : ! $test_build_rc;
-            ok( $is_rc_correct,
-                "$case->{name}: '$tool_label' returned " .
+            ok( $is_rc_correct, 
+                "$case->{name}: '$tool_label' returned " . 
                 $case->{"$tool\_success"}
             );
-
+            
             # Grade evaluation with special case if discarding
-            my ($found_grade_result, $found_msg) =
-                ( $stdout =~ /^CPAN::Reporter: (Test result[^,]+), ([^\n]+)[.:]$/ims );
+            my ($found_grade_result, $found_msg) = 
+                ( $stdout =~ /^CPAN::Reporter: ([^,]+), ([^\n]+)/ms );
             if ( $case->{"$tool\_grade"} eq 'discard' ) {
-                is ($found_grade_result, "test results were not valid",
+                is ($found_grade_result, "Test results were not valid",
                     "$case->{name}: '$tool_label' prerequisites not satisifed"
                 );
-
-                like( $stdout,
-                    "/Test report will not be sent/",
+                    
+                like( $stdout, 
+                    "/Test results for \Q$short_name\E will be discarded/",
                     "$case->{name}: discard message correct"
                 );
 
@@ -433,11 +388,11 @@ sub test_grade_test {
             else {
                 my $case_grade = $case->{"$tool\_grade"};
                 my ($found_grade) = ( $found_grade_result =~ /Test result is '([^']+)'/ );
-                is( $found_grade, $case_grade,
+                is( $found_grade, $case_grade, 
                     "$case->{name}: '$tool_label' grade reported as '$case_grade'"
                 ) or _diag_output( $stdout, $stderr );
-
-                like( $stdout, "/preparing a CPAN Testers report for \Q$short_name\E/",
+                
+                like( $stdout, "/Preparing a CPAN Testers report for \Q$short_name\E/",
                     "$case->{name}: report notification correct"
                 );
 
@@ -452,10 +407,9 @@ sub test_grade_test {
                     );
                 }
             }
-
+            
             # Grade explanation message
-            is( $found_msg,
-                $case->{"$tool\_msg"} ? $case->{"$tool\_msg"} : q{},
+            is( $found_msg, $case->{"$tool\_msg"} . ".",
                 "$case->{name}: '$tool_label' grade explanation correct"
             );
 
@@ -474,38 +428,27 @@ All tests were successful.
 HERE
 
     'fail' => <<'HERE',
-Thank you for uploading your work to CPAN.  However, there was a problem
-testing your distribution.
-
-If you think this report is invalid, please consult the CPAN Testers Wiki
-for suggestions on how to avoid getting FAIL reports for missing library
-or binary dependencies, unsupported operating systems, and so on:
-
-http://wiki.cpantesters.org/wiki/CPANAuthorNotes
+Thank you for uploading your work to CPAN.  However, it appears that
+there were some problems with your distribution.  If these results are 
+not what you expect, please consult "Notes for CPAN Authors" on 
+the CPAN Testers Wiki: http://cpantest.grango.org
 HERE
 
     'unknown' => << 'HERE',
 Thank you for uploading your work to CPAN.  However, attempting to
-test your distribution gave an inconclusive result.
-
-This could be because your distribution had an error during the make/build
-stage, did not define tests, tests could not be found, because your tests were
-interrupted before they finished, or because the results of the tests could not
-be parsed.  You may wish to consult the CPAN Testers Wiki:
-
-http://wiki.cpantesters.org/wiki/CPANAuthorNotes
+test your distribution gave an inconclusive result.  This could be because
+you did not define tests (or tests could not be found), because
+your tests were interrupted before they finished, or because
+the results of the tests could not be parsed by CPAN::Reporter.
 HERE
 
     'na' => << 'HERE',
-Thank you for uploading your work to CPAN.  While attempting to build or test
-this distribution, the distribution signaled that support is not available
-either for this operating system or this version of Perl.  Nevertheless, any
-diagnostic output produced is provided below for reference.  If this is not
-what you expect, you may wish to consult the CPAN Testers Wiki:
-
-http://wiki.cpantesters.org/wiki/CPANAuthorNotes
+Thank you for uploading your work to CPAN.  While attempting to build or test 
+this distribution, the distribution signaled that support is not available 
+either for this operating system or this version of Perl.  Nevertheless, any 
+diagnostic output produced is provided below for reference.
 HERE
-
+    
 );
 
 sub test_report_plan() { 13 };
@@ -517,52 +460,45 @@ sub test_report {
     my $expected_grade = $case->{expected_grade};
     my $prereq = CPAN::Reporter::_prereq_report( $case->{dist} );
     my $msg_re = $report_para{ $expected_grade };
-    my $default_comment = $ENV{AUTOMATED_TESTING}
-        ? "this report is from an automated smoke testing program\nand was not reviewed by a human for accuracy"
-        : "none provided" ;
 
     my $tempd = _ok_clone_dist_dir( $case->{name} );
-
+                
     my ($stdout, $stderr, $err, $test_output) = _run_report( $case );
-
-    is( $err, q{},
-        "$label report ran without error"
+    
+    is( $err, q{}, 
+        "$label report ran without error" 
     );
 
     ok( defined $msg_re && length $msg_re,
         "$label found '$expected_grade' grade paragraph"
     );
-
+    
     # set PERL_MM_USE_DEFAULT to mirror _run_report
     local $ENV{PERL_MM_USE_DEFAULT} = 1;
     my $env_vars = CPAN::Reporter::_env_report();
     my $special_vars = CPAN::Reporter::_special_vars_report();
     my $toolchain_versions = CPAN::Reporter::_toolchain_report();
-
+    
     like( $t::Helper::sent_report, '/' . quotemeta($msg_re) . '/ms',
         "$label correct intro paragraph"
     );
 
-    like( $t::Helper::sent_report, '/' . quotemeta($default_comment) . '/ms',
-        "$label correct default comment"
-    );
-
     like( $t::Helper::sent_report, '/' . quotemeta($prereq) . '/ms',
         "$label found prereq report"
     );
-
+    
     like( $t::Helper::sent_report, '/' . quotemeta($env_vars) . '/ms',
         "$label found environment variables"
     );
-
+    
     like( $t::Helper::sent_report, '/' . quotemeta($special_vars) . '/ms',
         "$label found special variables"
     );
-
+    
     like( $t::Helper::sent_report, '/' . quotemeta($toolchain_versions) . '/ms',
         "$label found toolchain versions found"
     );
-
+    
     my $joined_output = join("", @$test_output);
 
     # extract just the test output
@@ -573,14 +509,14 @@ sub test_report {
         ^(.+) \n                            # test output ending in newline
         ^------------------------------ \n  # separator
         ^PREREQUISITES \n                   # next section
-        /xms )
+        /xms ) 
     {
         $found_test_output = $1;
     }
-
+    
     my $orig_found_length = length $found_test_output;
     ok( $orig_found_length, "$label located test output in report" );
-
+    
     # if output appears longer than max, the excess should only be the
     # error message, so look for it, strip it and check it
     my $length_error = q{};
@@ -609,7 +545,7 @@ sub test_report {
     # after extracting error, if any, the output should now be
     # less than the max length
     my $found_length = length $found_test_output;
-    ok( $found_length <= $max_report_length,
+    ok( $found_length <= $max_report_length, 
         "$label test_output less than or equal to maximum length"
     ) or diag "length $found_length > $max_report_length";
 
@@ -623,7 +559,16 @@ sub test_report {
         "$label found output matches expected output"
     );
 
-    return ($stdout, $stderr, $err, $test_output);
+    my @expected_cc;
+    my $author = $case->{dist}->author;
+    push @expected_cc, $author->id if defined $author;
+    is_deeply( 
+        [ @t::Helper::cc_list ], 
+        [ map { $_ . '@cpan.org' } @expected_cc ],
+        "$label cc list correct"
+    );
+
+    return;
 };
 
 #--------------------------------------------------------------------------#
@@ -644,19 +589,18 @@ sub test_dispatch {
     my %opt = @_;
 
     my $tempd = _ok_clone_dist_dir( $case->{name} );
-    local $case->{dist}{build_dir} = "$tempd";
-
+                
     my ($stdout, $stderr, $err) = _run_report( $case );
 
-    is( $err, q{},
-            "generate report for $case->{label}"
+    is( $err, q{}, 
+            "generate report for $case->{label}" 
     );
 
     if ( $opt{will_send} ) {
         ok( defined $t::Helper::sent_report && length $t::Helper::sent_report,
             "report was sent for $case->{label}"
         );
-        like( $stdout, "/sending test report with/",
+        like( $stdout, "/Sending test report with/",
             "saw report sent message for $case->{label}"
         );
     }
@@ -678,7 +622,7 @@ sub test_dispatch {
 
 sub _diag_output {
     my ( $stdout, $stderr ) = @_;
-    diag "STDOUT:\n$stdout\n\nSTDERR:\n$stderr\n";
+    diag "STDOUT:\n$stdout\n\nSTDERR:\n$stderr\n"; 
 }
 
 #--------------------------------------------------------------------------#
@@ -688,8 +632,10 @@ sub _diag_output {
 sub _ok_clone_dist_dir {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my $dist_name = shift;
-    my $dist_dir = File::Spec->catdir( $temp_dist_dir, "dist", $dist_name );
-    my $work_dir = tempd()
+    my $dist_dir = File::Spec->rel2abs(
+        File::Spec->catdir( qw/t dist /, $dist_name )
+    );
+    my $work_dir = tempd() 
         or die "Couldn't create temporary distribution dir: $!\n";
 
     # workaround badly broken F::C::R 0.34 on Windows
@@ -717,11 +663,12 @@ sub _run_report {
 
     # automate CPAN::Reporter prompting
     local $ENV{PERL_MM_USE_DEFAULT} = 1;
-
+    
     my ($stdout, $stderr, $output, $exit_value);
-
+    
     $t::Helper::sent_report = undef;
     $t::Helper::comments = undef;
+    @t::Helper::cc_list = ();
 
     eval {
         capture sub {
@@ -732,7 +679,7 @@ sub _run_report {
             if ( $phase eq 'test' ) {
                 system("$make");
             }
-            ($output, $exit_value) =
+            ($output, $exit_value) = 
                 CPAN::Reporter::record_command( $case->{command} );
             no strict 'refs';
             &{"CPAN::Reporter::grade_$phase"}(
@@ -742,7 +689,7 @@ sub _run_report {
                 $exit_value,
             );
         } => \$stdout, \$stderr;
-    };
+    }; 
     if ( $@ ) {
         diag "DIED WITH:\n$@";
         _diag_output( $stdout, $stderr );
@@ -767,12 +714,17 @@ sub _short_name {
 #--------------------------------------------------------------------------#
 
 BEGIN {
+    $INC{"File/HomeDir.pm"} = 1; # fake load
     $INC{"Test/Reporter.pm"} = 1; # fake load
 }
 
+package File::HomeDir;
+sub my_documents { return $home_dir };
+sub my_home { return $home_dir };
+sub my_data { return $home_dir };
+
 package Test::Reporter;
-use vars qw/$AUTOLOAD $VERSION/;
-$VERSION = 999; # more than 1.54 (e.g. distfile exists)
+use vars qw/$AUTOLOAD/;
 
 sub new { return bless {}, 'Test::Reporter::Mocked' }
 
@@ -782,11 +734,12 @@ use vars qw/$AUTOLOAD/;
 
 sub comments { shift; $t::Helper::comments = shift }
 
-sub send {
-    shift;
-    $t::Helper::sent_report = $t::Helper::comments;
-    return 1
-}
+sub send { 
+    shift; 
+    $t::Helper::sent_report = $t::Helper::comments; 
+    @t::Helper::cc_list = ( @_ ); 
+    return 1 
+} 
 
 sub subject {
     my $self = shift;
@@ -795,20 +748,6 @@ sub subject {
 }
 
 my %mocked_data;
-
-my @valid_transport = qw/Metabase/;
-
-sub transport {
-    my ($self) = shift;
-    if (@_) {
-        my $t = shift;
-        die __PACKAGE__ . ":transport: '$t' is invalid\n"
-            unless grep { $t eq $_ } @valid_transport;
-        $mocked_data{transport} = $t;
-    }
-    return $mocked_data{transport};
-}
-
 sub AUTOLOAD {
     my $self = shift;
     if ( @_ ) {
